import { Hono } from "hono"
import { z } from "zod"
import { zValidator } from "@hono/zod-validator"
import { NotionProvider } from "./provider/notion"
import { BocProvider } from "./provider/boc"

type Bindings = {
  NOTION_KEY: string
  NOTION_DB_ID: string
}

const xRateQuerySchema = z.object({
  from: z
    .string()
    .min(3, "Currency code must at length 3.")
    .max(3, "Currency code must at length 3."),
  to: z
    .string()
    .min(3, "Currency code must at length 3.")
    .max(3, "Currency code must at length 3."),
  amount: z.string(),
})

const app = new Hono<{ Bindings: Bindings }>()

app.get(
  "/convert/:from/:to/:amount",
  zValidator("param", xRateQuerySchema, (result, c) => {
    if (!result.success) {
      return c.json({ error: result.error.errors }, 400)
    }
  }),
  async c => {
    const notionKey = c.env.NOTION_KEY
    const notionDatabaseId = c.env.NOTION_DB_ID

    if (!notionDatabaseId || !notionKey) {
      return c.json({ error: "Missing environment variables." }, 500)
    }

    const from = c.req.param("from")
    const to = c.req.param("to")
    const amount = parseFloat(c.req.param("amount"))

    try {
      const notion = new NotionProvider(notionKey, notionDatabaseId)
      const ratesMap = await notion.queryRates([from, to])
      return c.json(
        {
          rates: ratesMap,
          exchange_rate: ratesMap[to] / ratesMap[from],
          amount: amount,
          target: (amount * ratesMap[to]) / ratesMap[from],
        },
        200,
        {
          "Content-Type": "application/json; charset=utf-8",
          "Cache-Control": "s-maxage=300, stale-while-revalidate=60",
          "Access-Control-Allow-Origin": "*",
        }
      )
    } catch (error: any) {
      console.log(error)
      return c.json({ error: error.message }, 500)
    }
  }
)

app.get("/boc", async c => {
  const boc = new BocProvider()
  return c.json({
    resp: await boc.queryRates(),
  })
})

export default app
